package game;

import java.util.concurrent.Callable;

/**
 * This BasicPlayer class implements the basic functions of a player in the
 * algorithm guessing game. It implements the basic decision making, but
 * leaves the specific strategy to the subclasses.
 */
public abstract class BasicPlayer implements Callable<String> {
  /**
   * This GameBoard is where each Player can find the latest guess.
   */
  private GameBoard theBoard;
 
  /**
   * Remember the game board object where the dealer's guesses are
   * displayed.
   * @param board the GameBoard that holds information about the current state
   *          of the game (namely, the most recent guess)
   */
  public BasicPlayer(GameBoard board) {
    theBoard = board;
  }

  /**
   * The call() method provides the overall decision making
   * capability for any player.
   */
  public String call() throws Exception {
    String resultString = this.toString()+" default result string...";
    try {
      System.out.println("Player " + this + " is at the table.");
      
      // stay at the table until we get a number we like

      boolean looking = true;
      synchronized (theBoard) {
        while (looking) {
          theBoard.decrementLatch();  // we are ready for a new number
          theBoard.wait();
          int value = theBoard.getGuess();
          if (isAcceptable(value)) {
            System.out.println("Player " + this + " finds that "
                + value + " is acceptable.");
            looking = false;
            resultString = this.toString()+" accepted "+value;
          } else {
            System.out.println("Player " + this + " finds that "
                + value + " is not acceptable.");
          }
        }
      }
      System.out.println("Player " + this + " has left the table.");
    } catch (InterruptedException e) {
      resultString = this.toString() + 
          " was interrupted and never accepted a value.";
    }
    return resultString;
  }

  /**
   * Evaluate the guessed number and decide if it's okay according to our
   * algorithm.
   * @param n the number to evaluate
   * @return true if okay, false if not
   */
  public abstract boolean isAcceptable(int n);
  /**
   * Return a String containing information about this object and the thread
   * that it is running in.
   */
  public String toString() {
    Thread t = Thread.currentThread();
    return super.toString() + " (Thread "+t.getId()+", "+t.getName()+")";
  }
}
